`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/22 19:35:26
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

module topmodule(
    clk,
    rst,
    pb_in,
    vgaRed,
    vgaGreen,
    vgaBlue,
    hsync,
    vsync
);

    input clk; // clock from the crystal
    input rst; 
    input pb_in;
    output [3:0] vgaRed, vgaGreen, vgaBlue;
    output hsync, vsync;

    wire clk_25M, clk_22;
    wire pb_in_debounced, pb_out_pulse;
    wire pb_rst_debounced;
    wire count_en;
    wire hsync, vsync, valid;
    wire [9:0] h_cnt, v_cnt;
    wire [16:0] pixel_addr;

    wire [11:0] data;
    wire [11:0] pixel;

    assign {vgaRed, vgaGreen, vgaBlue} = (valid == 1'b1) ? pixel : 12'h0;

    clock_divisor U0(
    .clk1(clk_25M), 
    .clk(clk), 
    .clk22(clk_22)
    );

    debounce U1( 
    .rst(pb_rst_debounced), 
    .clk(clk), 
    .pb_in(pb_in),
    .pb_debounced(pb_in_debounced)
);
    debounce U11( 
    .rst(pb_rst_debounced), 
    .clk(clk), 
    .pb_in(rst),
    .pb_debounced(pb_rst_debounced)
);
    one_pulse U2( 
    .rst(pb_rst_debounced), 
    .clk(clk), 
    .in_trig(pb_in_debounced),
    .out_pulse(pb_out_pulse)
);
    fsm U3(
    .select_in(pb_out_pulse),
    .clk(clk),
    .curr_state(count_en),
    .rst(pb_rst_debounced)
);
    vga_controller U4(
    .pclk(clk_25M),
    .reset(pb_rst_debounced),
    .hsync(hsync),
    .vsync(vsync),
    .valid(valid),
    .h_cnt(h_cnt),
    .v_cnt(v_cnt)
    );

    mem_addr_gen U5(
    .clk(clk_22),
    .rst(pb_rst_debounced),
    .en(count_en),
    .h_cnt(h_cnt),
    .v_cnt(v_cnt),
    .pixel_addr(pixel_addr)
);
    blk_mem_gen_0 U6(
    .clka(clk_25M),
    .wea(0),
    .addra(pixel_addr),
    .dina(data[11:0]),
    .douta(pixel)
);

    

    
endmodule



